// Copyright (c) 2023 Tudor Stanciu

import axios from "axios";
import { localStorage } from "@flare/js-utils";
import { storageKeys } from "./constants";
import { getUrlTemplates } from "./config";
import { isValidURL } from "./utils";
import type { UrlTemplates } from "./config";

const { setItem, getItem, removeItem } = localStorage;

async function request(url: string, method: string) {
  try {
    const res = await axios.request({ url, method });
    return res.data;
  } catch (error: any) {
    if (error.response && error.response.data) {
      const { detail, title } = error.response.data;
      throw {
        ...error.response.data,
        message: detail || title
      };
    }
    throw error;
  }
}

export type TuitioLoginResult = { result: { token: string; expiresIn: number }; error: string | null };
export type TuitioLogoutResult = { result: { userId: number; userName: string; logoutDate: Date }; error: string };

class TuitioClient {
  tuitioUrl: string;
  urlTemplates: UrlTemplates;

  constructor(tuitioUrl: string | null) {
    if (tuitioUrl === null || tuitioUrl === "" || isValidURL(tuitioUrl) === false) {
      throw new Error(`TuitioClient: ${tuitioUrl} is not a valid URL.`);
    }

    this.tuitioUrl = tuitioUrl;
    this.urlTemplates = getUrlTemplates(this.tuitioUrl);
  }

  async login(userName: string, password: string): Promise<TuitioLoginResult> {
    const url = this.urlTemplates.login.replace("{username}", userName).replace("{password}", password);

    const response = await request(url, "post");
    if (!response.error) {
      const currentDate = new Date();
      const expiresIn = response.result.expiresIn - 10000; //10 seconds of safety margin
      const validUntil = new Date(currentDate.getTime() + expiresIn);
      const authData = { token: response.result.token, validUntil, userName };
      setItem(storageKeys.AUTH_DATA, authData);
    }

    return response;
  }

  async logout(token: string): Promise<TuitioLogoutResult> {
    const url = this.urlTemplates.logout.replace("{token}", token);
    const response = await request(url, "post");
    if (!response.error) {
      removeItem(storageKeys.AUTH_DATA);
    }
    return response;
  }
}

export type TuitioState = { token: string | null; validUntil: Date | null; userName: string | null };
const defaultTuitioState: TuitioState = { token: null, validUntil: null, userName: null };

const fetch = (): TuitioState => {
  const authData = getItem(storageKeys.AUTH_DATA);
  if (!authData) return defaultTuitioState;
  const { token, validUntil, userName } = authData;
  const data: TuitioState = {
    token,
    validUntil,
    userName
  };
  return data;
};

export { TuitioClient, fetch };
export default TuitioClient;
